var stream = require('stream')
var util = require('util')
var isBuffer = require('isbuffer')

function WebsocketStream(server, options) {
  if (!(this instanceof WebsocketStream)) return new WebsocketStream(server, options)
  stream.Stream.call(this)
  this.options = options || {}
  this.readable = true
  this.writable = true
  if (typeof server === "object") {
    this.ws = server
    this.ws.on('message', this.onMessage.bind(this))
    this.ws.on('error', this.onError.bind(this))
    this.ws.on('close', this.onClose.bind(this))
    this.ws.on('open', this.onOpen.bind(this))
  } else {
    this.ws = new WebSocket(server, this.options.protocol)
    this.ws.binaryType = this.options.binaryType || 'arraybuffer'
    this.ws.onmessage = this.onMessage.bind(this)
    this.ws.onerror = this.onError.bind(this)
    this.ws.onclose = this.onClose.bind(this)
    this.ws.onopen = this.onOpen.bind(this)
  }
}

util.inherits(WebsocketStream, stream.Stream)

module.exports = WebsocketStream
module.exports.WebsocketStream = WebsocketStream

WebsocketStream.prototype.onMessage = function(e, flags) {
  if (e.data) return this.emit('data', e.data, flags)
  this.emit('data', e, flags)
}

WebsocketStream.prototype.onError = function(err) {
  this.emit('error', err)
}

WebsocketStream.prototype.onClose = function(err) {
  this.emit('end')
}

WebsocketStream.prototype.onOpen = function(err) {
  this.emit('open')
  this.emit('connect')
}

WebsocketStream.prototype.write = function(data) {
  typeof WebSocket != 'undefined' && this.ws instanceof WebSocket
    ? this.ws.send(data)
    : this.ws.send(data, { binary : isBuffer(data) })
}

WebsocketStream.prototype.end = function() {
  this.ws.close()
}
